require_relative "../lib/helpers/quote_id_helper"

module CreateRecommendationTests
  RSpec.shared_examples 'create and verify new recommendation' do |variables|
    include_context 'request metadata'
    include QuoteIdHelper

    it 'a new quote for a single account can be created and retrieved' do
      new_quote_id = "apitest_quote_#{rand_quote_id}-skip-completed-skip-waiting"
      variables[:input][:quoteId] = new_quote_id
      accountId = variables[:input][:accountIds][0]
      get_api_response(variables, :createSpRecommendation)
      new_quote = json_body[:data][:createSpRecommendation]

      expect(new_quote[:status]).to eq('CREATED')
      expect(new_quote[:accountId].to_s).to eq(variables[:customerInfo][:accountId].to_s)
      expect(new_quote[:quoteId].to_s).to eq(new_quote_id)
      # wait for quote creation to complete before verification
      job_completion_check('SpRecommendation', new_quote[:id], 19)

      # retrieve newly created quote via the spRecommendations route
      # verify quote completion and data consistency
      sp_recommendations_variables = {
          customerInfo: CUSTOMER_INFO,
          quoteId: new_quote_id
      }
      get_api_response(sp_recommendations_variables, :spRecommendations)
      expect(json_body[:data][:spRecommendations][:edges].length).to eq(1)
      retrieved_quote = json_body[:data][:spRecommendations][:edges][0][:node]

      # the following values can only be verified on quote completion
      expect(retrieved_quote[:finishedAt]).not_to be_nil
      expect(Time.parse(retrieved_quote[:finishedAt])).to be > Time.parse(new_quote[:createdAt])
      verify_saved_quote(new_quote, retrieved_quote, accountId)

      # re run the exact same quote creation request, ensure existing results are reused
      no_skip_quote_id = "apitest_quote_#{rand_quote_id}-no-skip"
      variables[:input][:quoteId] = no_skip_quote_id
      get_api_response(variables, :createSpRecommendation)

      new_quote = json_body[:data][:createSpRecommendation]
      expect(new_quote[:status]).to eq('COMPLETED')
      expect(new_quote[:accountId].to_s).to eq(variables[:customerInfo][:accountId].to_s)
      expect(new_quote[:quoteId].to_s).to eq(no_skip_quote_id)
      job_completion_check('SpRecommendation', new_quote[:id], 19)

      no_skip_sp_recommendations_variables = {
        customerInfo: CUSTOMER_INFO,
        quoteId: no_skip_quote_id
      }
      get_api_response(no_skip_sp_recommendations_variables, :spRecommendations)
      expect(json_body[:data][:spRecommendations][:edges].length).to eq(1)
      retrieved_quote = json_body[:data][:spRecommendations][:edges][0][:node]
      expect(retrieved_quote[:status]).to eq('COMPLETED')
      expect(retrieved_quote[:finishedAt]).not_to be_nil
      expect(Time.parse(retrieved_quote[:finishedAt])).to be >= Time.parse(new_quote[:createdAt])
      verify_saved_quote(new_quote, retrieved_quote, accountId)
    end
  end

  RSpec.shared_examples 'Waiting state job verification' do |variables|
    include QuoteIdHelper

    # this method wait and verifies waiting job goes to completion
    # after primary job completes, without going to RUNNING STATE
    def quote_waiting_check(principal_quote_id, waiting_quote_id)
      # Currently, the test allows 3 minutes for quote creation to COMPLETE
      # as test jobs typically stay running for 30-40 seconds, a status check
      # of every 5 seconds should be enough to verify RUNNING state isn't entered
      wait_time_increment = 5
      retry_attempts = 36
      time_for_complete = wait_time_increment * (retry_attempts - 1)
      principal_sp_variables = {
          customerInfo: CUSTOMER_INFO,
          quoteId: principal_quote_id
      }
      waiting_sp_variables = {
          customerInfo: CUSTOMER_INFO,
          quoteId: waiting_quote_id
      }
      puts "getting intial status for waiting quote #{waiting_quote_id}"
      get_api_response(waiting_sp_variables, :spRecommendations)
      waiting_quote_status = json_body[:data][:spRecommendations][:edges][0][:node][:status]
      if waiting_quote_status == 'FAILED'
        raise Exception, "recommendation creation job failed: #{json_body}"
      end

      attempts = 0
      wait_time = wait_time_increment
      while waiting_quote_status != 'COMPLETED' && attempts < retry_attempts
        sleep(wait_time_increment)
        wait_time += wait_time_increment
        puts "waiting #{wait_time} seconds for create quote completion (quote id #{waiting_quote_id}, status #{waiting_quote_status})"
        get_api_response(waiting_sp_variables, :spRecommendations)
        waiting_quote_status = json_body[:data][:spRecommendations][:edges][0][:node][:status]
        # fail fast if waiting quote status goes to RUNNING
        get_api_response(principal_sp_variables, :spRecommendations)
        principal_quote_status = json_body[:data][:spRecommendations][:edges][0][:node][:status]
        # waiting quote should stay in CREATED state while primary running job is processing
        if principal_quote_status != 'COMPLETED'
          expect(waiting_quote_status).to eq('CREATED')
        end
        # fail fast if quote status goes to RUNNING or FAILED
        if waiting_quote_status == 'RUNNING'
          raise Exception, "waiting quote got to RUNNING state: #{json_body}"
        elsif waiting_quote_status == 'FAILED'
          raise Exception, "waiting recommendation creation job failed: #{json_body}"
        end
        attempts += 1
      end

      if waiting_quote_status != 'COMPLETED'
        raise Exception, "Quote creation not COMPLETED in #{time_for_complete} seconds, status: #{waiting_quote_status}"
      end
      check(:spRecommendations, service: service)
    end

    it 'Verify waiting quote completes when primary quote completes, and never enters RUNNING state' do
      primary_quote_id = "apitest_quote_#{rand_quote_id}-skip-completed-skip-waiting"
      variables[:input][:quoteId] = primary_quote_id
      accountId = variables[:input][:accountIds][0]
      get_api_response(variables, :createSpRecommendation)
      primary_quote = json_body[:data][:createSpRecommendation]
      expect(primary_quote[:status]).to eq('CREATED')
      # wait 2 seconds before creating 'waiting' quote
      sleep(2)

      waiting_quote_id = "apitest_quote_#{rand_quote_id}-skip-completed"
      variables[:input][:quoteId] = waiting_quote_id
      get_api_response(variables, :createSpRecommendation)
      waiting_quote = json_body[:data][:createSpRecommendation]
      expect(waiting_quote[:status]).to eq('CREATED')

      quote_waiting_check(primary_quote_id, waiting_quote_id)

      waiting_quote_variables = {
        customerInfo: CUSTOMER_INFO,
        quoteId: waiting_quote_id
      }
      get_api_response(waiting_quote_variables, :spRecommendations)
      retrieved_waiting_quote = json_body[:data][:spRecommendations][:edges][0][:node]
      verify_saved_quote(waiting_quote, retrieved_waiting_quote, accountId)

      primary_quote_variables = {
        customerInfo: CUSTOMER_INFO,
        quoteId: primary_quote_id
      }
      get_api_response(primary_quote_variables, :spRecommendations)
      retrieved_principal_quote = json_body[:data][:spRecommendations][:edges][0][:node]
      expect(retrieved_principal_quote[:status]).to eq('COMPLETED')
      verify_saved_quote(primary_quote, retrieved_principal_quote, accountId)
      expect(Time.parse(retrieved_waiting_quote[:finishedAt])).
          to be >= Time.parse(retrieved_principal_quote[:finishedAt])
    end
  end
end
