require 'cht-chaphql'
require 'cht_apitest_local_auth'
require 'cht_apitest_local_auth/service'
require './shared_tests/create_recommendation_tests'

Dir[File.join(__dir__, '../lib/*', '*.rb')].each { |file| require file }
Dir[File.join(__dir__, 'helpers/*.rb')].each { |f| require f }

RSpec.configure do |c|
  c.include Chaphql
  c.include Chaphql::GraphQLHelpers
  c.include Chaphql::AnalyzerHelpers
  c.include Chaphql::StringManipulation
  c.include Chaphql::ServiceHelper
  c.include Chaphql::IntrospectConflict
  c.include ChtApitestLocalAuth
  c.include ChtApitestAuthCommon
  c.include Spaceborne
  include ChtApitestSpAllocationScheduler
  include SpAllocationSchedulerHelpers
  c.filter_run_excluding(save_schema: true)
  c.filter_run_excluding(staging_env: true) unless ENV['TEST_ENV'] == 'staging'
  c.filter_run_excluding(not_via_proxy: true) if ENV['EKS_ENV'] == 'external'
  c.shared_context_metadata_behavior = :apply_to_host_groups
  c.before(:each) do |example|
    @test_tag = example.to_s
  end
  c.after(:suite) do
    puts Characterizer.check_results +
         Characterizer.test_missing_supported_data(
           IAnalyzer.new(nil).do_introspect(
             ChtApitestSpAllocationScheduler::SpAllocationSchedulerService.new,
             LocalAuthService.new(via_public: %w{prod preprod canary staging}.include?(getenv)))).join("\n")
  end
  c.example_status_persistence_file_path  = 'rspec_times.txt'
end

RSpec.shared_context 'request metadata', shared_context: :metadata do
  let(:via_public) { %w{prod preprod staging canary}.include?(getenv) }
  let(:service) { ChtApitestSpAllocationScheduler::SpAllocationSchedulerService.new }
  let(:sp_api_service) { SavingsPlansService.new(via_public: via_public) }
  let(:auth_service) { LocalAuthService.new(via_public: via_public) }
  let(:tokens) { login_api(ENV['API_KEY'], service: auth_service) }
  let(:headers) do
    apitest_headers(with_auth: true, access_token: tokens[:access_token])
  end
  let(:dbg_headers) { headers.merge(use_curl: 'debug') }
  let(:get_new_tokens) do
    headers = apitest_headers(service(LocalAuthService.new,
                                      with_auth: true,
                                      refresh: tokens[:refresh_token]))
    dbg_headers = headers.merge(use_curl: 'debug')
  end
end

RSpec.configure do |c|
  c.include_context 'request metadata', include_shared: true
end

def get_api_response(variables, request_type)
  wrap_request do
    graphql url(service: service),
      format_request(
        request_type,
        service: service
      ),
      variables, headers
    expect_success_graphql
    check(request_type, service: service)
    if response.code==200 && json_body.nil?
      raise Exception, 'api request returned 200 code and nil response'
    elsif response.code==200 && json_body.empty?
      raise Exception, 'api request returned 200 code and empty response'
    elsif response.code==200 && json_body[:data].nil?
      raise Exception, 'api request returned 200 code and response with nil data node'
    end
  end
end

def ensure_quote_presence(required_quote_number, account_id)
  variables = {customerInfo: CUSTOMER_INFO}
  get_api_response(variables, :spRecommendations)
  quote_count = json_body[:data][:spRecommendations][:edges].length

  # If for some reason there is no available quote associated with
  # the account being used for test, create one
  while (quote_count < required_quote_number)
    variables = {
      customerInfo: CUSTOMER_INFO,
      input: {
        accountIds: [account_id],
        evaluationPeriod: ['THIRTY_DAYS', 'SEVEN_DAYS'].sample,
        paymentOption: ['ALL_UPFRONT', 'PARTIAL_UPFRONT', 'NO_UPFRONT'].sample,
        term: ['ONE_YEAR', 'THREE_YEAR'].sample,
        purchaseDate: "2020-02-30T00:00:00Z"
      }
    }
    get_api_response(variables, :createSpRecommendation)
    if json_body[:data][:createSpRecommendation][:id].nil?
      raise Exception, 'No quotes available for test customer (and quote creation not successful)'
    end
    get_api_response(variables, :spRecommendations)
    quote_count = json_body[:data][:spRecommendations][:edges].length
  end
end

# method to wait a set amount of time until a quote is completed or failed
def job_completion_check(job_type, job_id, retry_attempts)
  # the time to allow for a quote to get to COMPLETED status
  # is a combination of the wait and retry values (retries * 10 seconds)
  wait_time_increment = 10
  time_for_complete = wait_time_increment * retry_attempts
  puts "getting initial status for job #{job_id}"
  wrap_request do
    variables = {}
    graphql url(service: service),
      format_request(
        :node,
        service: service,
        crn: job_id,
        type: job_type,
        fields: 'id, status'
      ),
      variables, headers
  end

  quote_status = json_body[:data][:node][:status]
  flexreport_setup_ran = false
  if quote_status == 'FAILED'
    raise Exception, "recommendation creation job failed: #{json_body}"
  elsif quote_status == 'FLEXREPORT_SETUP'
    flexreport_setup_ran = true
  end

  attempts = 0
  wait_time = wait_time_increment
  while quote_status != 'COMPLETED' && attempts < retry_attempts
    sleep(wait_time_increment)
    wait_time += wait_time_increment
    puts "waiting #{wait_time} seconds for create quote completion (job id: #{job_id}, status: #{quote_status})"
    wrap_request do
      variables = {}
      graphql url(service: service),
        format_request(
          :node,
          service: service,
          crn: job_id,
          type: job_type,
          fields:  'id, status'
        ),
        variables, headers
    end
    quote_status = json_body[:data][:node][:status]
    # fail fast if quote status goes to FAILED
    if quote_status == 'FAILED'
      raise Exception, "recommendation creation job failed: #{json_body}"
    end
    if quote_status == 'FLEXREPORT_SETUP'
      flexreport_setup_ran = true
    end
    attempts += 1
  end
  if quote_status != 'COMPLETED'
    raise Exception, "Quote #{job_id} did not COMPLETE in #{time_for_complete} seconds, status: #{quote_status}"
  end

  if job_type == 'ArtifactRefreshTaskStatus'
    expect(flexreport_setup_ran).to eq(true)
  else
    expect(flexreport_setup_ran).to eq(false)
  end
end

# common method to test saved quote verification for each different
# createSpSavedRecommendations request type in tests below
# the following values can only be verified on quote completion
def verify_saved_quote(new_quote, retrieved_quote, accountId)
  expect(retrieved_quote[:status]).to eq('COMPLETED')
  expect(retrieved_quote[:createdAt]).to eq(new_quote[:createdAt])
  expect(retrieved_quote[:accountId]).to eq(new_quote[:accountId])
  expect(retrieved_quote[:id]).to eq(new_quote[:id])
  expect(retrieved_quote[:customerId]).to eq(new_quote[:customerId])
  expect(retrieved_quote[:quoteId]).to eq(new_quote[:quoteId])

  expect(retrieved_quote[:exception]).to be_nil
  expect(retrieved_quote[:manifestS3Key]).not_to be_nil
  expect(retrieved_quote[:manifestS3path]).not_to be_nil
  expected_manifest_s3_key =
      "sp_discount_curve/#{CUSTOMER_INFO[:customerId]}/#{retrieved_quote[:id].split('/')[-1]}/recommendation-summary.json"

  # we expect the consolidated redshirt account to always have aws assets for quote creation
  # the linked account used currently doesn't, but we want to account for that changing in the future
  if accountId == TEST_CUSTOMER[:owner_id]
    expect(retrieved_quote[:manifestS3Key]).to eq(expected_manifest_s3_key)
    expect(retrieved_quote[:manifestS3path]).
        to eq("s3://cht-sp-analysis-use2-#{ENV['TEST_ENV']}-test/#{expected_manifest_s3_key}")
  else
    expect([expected_manifest_s3_key, "NO_DATA"]).to include(retrieved_quote[:manifestS3Key])
    expect(["s3://cht-sp-analysis-use2-#{ENV['TEST_ENV']}-test/#{expected_manifest_s3_key}", "NO_DATA"]).
        to include(retrieved_quote[:manifestS3path])
    # this is just to help spot check state of linked account used for test
    if retrieved_quote[:manifestS3path] != "NO_DATA"
      puts "linked account used in test (#{accountId}) now has data associated with it"
    end
  end
end

def get_sp_term_as_int(term)
  if term == 'THREE_YEAR'
    3
  else
    1
  end
end

TEST_CUSTOMER = {
  :aws_billing_id =>  3367254377911,
  :customer_id => 2017,
  :organization_id => 893353197629,
  :owner_id => '844584796563',
  :name => 'Redshirt'
}

TEST_CUSTOMER_LINKED_ACCOUNT = {
  :aws_billing_id =>  3367254362284,
  :customer_id => 2017,
  :organization_id => 893353197629,
  :owner_id => '081309885503',
  :name => 'Redshirt 2'
}

TEST_CUSTOMER_PREPROD = {
  :aws_billing_id =>  3,
  :customer_id => 57534,
  :organization_id => 5841155524188,
  :owner_id => '844584796563',
  :name => 'Redshirt'
}


TEST_CUSTOMER_LINKED_ACCOUNT_PREPROD = {
  :aws_billing_id =>  8,
  :customer_id => 57534,
  :organization_id => 5841155524188,
  :owner_id => '781971168076',
  :name => 'Redshirt 3'
}

TEST_DATA_ENV = if ENV['TEST_ENV'] == 'preprod'
                  "preprod"
                elsif ENV['TEST_ENV'] == 'staging'
                  "staging"
                else
                  'default'
                end

CUSTOMER_INFO= {
  accountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
  customerId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:customer_id] : TEST_CUSTOMER[:customer_id],
  organizationId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:organization_id] : TEST_CUSTOMER[:organization_id]
}
